@@ -25,10 +25,11 @@ from matplotlib.legend_handler import HandlerLine2D
 import seaborn as sns
 import xlsxwriter
 import datetime
+import shutil
+from scipy import stats
+#TEST
 
-
-
-__version__ = "0.1.1"
+__version__ = "0.1.5"
 
 
 
@@ -52,17 +53,20 @@ def argument_parser(argv_list=None):
 	parser.add_argument("-f", "--figures", help="Get only the growth curve figures", action = "store_true")
 	parser.add_argument("-s", "--summary", help="Get only the summary of growth rate estimations",action = "store_true")
 	parser.add_argument("-it", "--interpolation", help="Get interpolation of growth rate measurements with given od", action = "store_true")
+	parser.add_argument("-r", "--resultsdir", help="Name results directory",action='store')
+	parser.add_argument("-p", "--path", help="Path to data",action='store')
+
 
 	#Visualization arguments
 	
 	parser.add_argument("-b", "--bioshaker", help="Get one growth rate figure for every individual bioshaker", action = "store_true")
 	parser.add_argument("-i", "--individual", help="Get one growth rate figure for every individual sample", action = "store_true")
 	parser.add_argument("-bc", "--bioshakercolor", help="Get one growth rate figure for every species colored by bioshaker", action = "store_true")
-
+	parser.add_argument("-ip", "--interpolationplot", help="Shows interpolation between points on growth rate curves",action='store_true')
 
 	#Volume loss related arguments
 	
-	parser.add_argument("-v", "--volumeloss", help="Volume loss compesation is not computed", action = "store_false")
+	parser.add_argument("-v", "--novolumeloss", help="Volume loss compesation is not computed", action = "store_false")
 	parser.parse_args()
 	args = parser.parse_args(argv_list[1:])
 
@@ -77,9 +81,12 @@ def argument_parser(argv_list=None):
 		flag_fig = False
 		flag_ind = args.individual
 		flag_bioshakercolor = args.bioshakercolor
-		flag_volume_loss = args.volumeloss
+		flag_volume_loss = args.novolumeloss
 		flag_bioshaker = args.bioshaker
 		flag_interpolation = args.interpolation
+		cmd_dir = args.resultsdir
+		path = args.path
+		interpolationplot = args.interpolationplot
 
 	
 	elif args.estimations == True or args.figures == True or args.summary == True :
@@ -90,15 +97,76 @@ def argument_parser(argv_list=None):
 		flag_fig = args.figures
 		flag_ind = args.individual
 		flag_bioshakercolor = args.bioshakercolor
-		flag_volume_loss = args.volumeloss
+		flag_volume_loss = args.novolumeloss
 		flag_bioshaker = args.bioshaker
 		flag_interpolation = args.interpolation
+		cmd_dir = args.resultsdir
+		path = args.path
+		interpolationplot = args.interpolationplot
 
 
+	return flag_all, flag_est, flag_sum, flag_fig, flag_ind, flag_bioshakercolor, args.novolumeloss, flag_bioshaker, flag_interpolation, cmd_dir, path, interpolationplot
+
+# ------ INTERPRET INPUT ARGUMENTS AND CREATE OUTPUT DIR 
+
+def input_output(cmd_dir, path):
+	'''Interprets input arguments related to the path to the data and output directory
+	Args:
+	cmd_dir : Name of directory where output will be sent
+	path : path where the data is
+	'''
+
+	# Interpret path input
+	if path is None :
+		pass
+
+	else :
+
+		try :
+			os.chdir(str(path))
+
+		except :
+			sys.exit("Entered path does not exist: "+str(path))
 
 
-	return flag_all, flag_est, flag_sum, flag_fig, flag_ind, flag_bioshakercolor, args.volumeloss, flag_bioshaker, flag_interpolation
+	# Output directory naming
 
+	if cmd_dir is None :
+
+		if os.path.exists("Results") == True :
+			shutil.rmtree('Results', ignore_errors=True)
+
+		else :
+			pass
+
+		try:
+			os.mkdir("Results")
+			print ("Successfully created the Results directory")
+			
+
+		except OSError:
+			sys.exit("Error! Creation of the directory failed")
+
+		dir_ = "Results"
+
+	else :
+
+		if os.path.exists(cmd_dir) == True :
+			shutil.rmtree(cmd_dir, ignore_errors=True)
+
+		else :
+			pass
+
+		try:
+			os.mkdir(str(cmd_dir))
+			print (f"Successfully created the {cmd_dir} directory")
+
+		except OSError:
+			sys.exit("Error! Creation of the directory failed")
+
+		dir_ = str(cmd_dir)
+
+	return dir_
 
 
 # ------ PARSE THE DATA USING THE autoflow_parser LIBRARY ------
@@ -150,13 +218,43 @@ def sample_outcome(sample_file, df) : #done
 
 	#Open the file containing sample purposes
 	df_calc = pd.read_csv(sample_file, sep="\t")  #Info about the purpose of the sample (growth rate, volume loss compensation, species and drop-out samples)
-	df_calc = df_calc[df_calc.Drop_out == False]
 	
+	# Check format consistency across files
+	IDs_parsed = df["Sample_ID"].tolist()
+	IDs_calc = df_calc["Sample_ID"].tolist()
+
+	# Check that two random identifiers are in the calc file
+	if IDs_parsed[0] not in IDs_calc and IDs_parsed[10] not in IDs_calc:
+
+		print(f"""Warning: The format of the Sample IDs is not matching across the input files:
+			\n
+			results.xlsx : {IDs_parsed[0]}
+			calc.tsv : {IDs_calc[0]}\n
+			""")
+		
+		if re.search(r"BS\d[.]\d_[A-Z]\d", IDs_calc[0]) is not None :
+			df_calc["Sample_ID"] = df_calc["Sample_ID"].str.replace(".0_", "_")
+			print("The ID format will be redefined to match the results file.")
+			print(f'e.g {IDs_calc[0]} --> {df_calc["Sample_ID"].tolist()[0]}')
+
+		elif re.search(r"BS\d[.]\d_[A-Z]\d", IDs_calc[0]) is None:
+			df_calc["Sample_ID"] = df_calc["Sample_ID"].str.replace("_", ".0_")
+			print("The ID format will be redefined to match the results file.")
+			print(f'e.g {IDs_calc[0]} --> {df_calc["Sample_ID"].tolist()[0]}')
+
+		else :
+			sys.exit("The ID format of the calc.tsv file could not be converted to match the results.xlsx file")
+
+
+	# Remove wells to drop
+	df_calc = df_calc.loc[df_calc["Drop_out"] == False]
+
 	#Add species and bioshaker labels to every observation
-	cols=["Sample_ID", "Species"]
+	cols=["Sample_ID", "Species", "Dilution"]
 	temp_df = df_calc[cols]
 	df = pd.merge(df, temp_df, how="left", on="Sample_ID")
 	df["bioshaker"] = df["Sample_ID"].str[0:3]
+	df["Measurement"] = df["Measurement"] * df["Dilution"]
 
 	#Separate samples for growth rate or volume loss according to calc.tsv
 	gr_samples = df_calc.loc[df_calc["calc_gr"] == True]
@@ -207,15 +305,20 @@ def time_formater(df) :
 
 		#Substracting the time of the first obs to all obs
 		df_temp['time_hours'] = df_temp["date_time"] - df_temp.loc[df_temp.index[0], 'date_time']
-		df_temp["time_hours"] = df_temp["time_hours"].dt.total_seconds()/3600
+		df_temp['h'] = df_temp['time_hours'].dt.components['hours']
+		df_temp['m'] = df_temp['time_hours'].dt.components['minutes']
+		df_temp['s'] = df_temp['time_hours'].dt.components['seconds']
+		df_temp["time_hours"] = df_temp['h']+df_temp['m']/60+df_temp['s']/360
+
+		#df_temp["time_hours"] = df_temp["time_hours"].dt.total_seconds()/3600
 
 		#Append dataframes together
 		df_out = df_out.append(df_temp)
 	
 	#Removal of temporary variables
 	df_out = df_out.drop_duplicates()
-	df_out = df_out.drop(columns=["Sampling_date", "Sampling_time"])
-	df_out = df_out.drop(columns=["date_time"])
+	#df_out = df_out.drop(columns=["Sampling_date", "Sampling_time"])
+	#df_out = df_out.drop(columns=["date_time"])
 	
 	return df_out
 
@@ -246,6 +349,7 @@ def vol_correlation(df_vl): #done
 		df_vl_ID["Correlation"] = df_vl_ID["Measurement"]/init_val
 		cor_df = cor_df.append(df_vl_ID)
 		df_vl_ID = pd.DataFrame()
+
 	
 	return cor_df
 
@@ -271,8 +375,7 @@ def compensation_lm(cor_df, df_gr) : #done
 	linear = lambda x, a, b: a * x + b
 	sns.set(style="white", palette="muted", color_codes=True)
 	fig = plt.figure()
-	fig.suptitle('Linear models of volume loss correlation to time for different plates', fontweight="bold")
-	#sns.despine(left=True)
+	fig.suptitle('Volume loss correlation over time', fontweight="bold")
 	lm_eq = []
 
 	
@@ -284,16 +387,13 @@ def compensation_lm(cor_df, df_gr) : #done
 		lm_eq.append(popt)
 		
 		ax = fig.add_subplot(math.ceil(len(unique_bioshaker)), math.ceil(len(unique_bioshaker)/2), shaker+1)
-		#ax.get_xaxis().set_visible(True)
-		#ax.get_yaxis().set_visible(True)
 		ax.plot(sub_cor_df["time_hours"], sub_cor_df["Correlation"], "o", label="Empirical data", markerfacecolor='skyblue', markeredgecolor="dodgerblue", alpha= 0.5)
 		ax.plot(sub_cor_df["time_hours"], linear(sub_cor_df ["time_hours"], *popt), "b-", label="Linear model", color = "darkred")
-		ax.set_ylabel('Volume loss correlation', labelpad=10)
+		ax.set_ylabel('Correlation', labelpad=10)
 		ax.set_xlabel('Time (h)', labelpad=10)
-		ax.set_title("\n"+unique_bioshaker[shaker])
-		plt.legend(bbox_to_anchor=(1.05, 1.0), loc='upper left')
+		ax.set_title("\n"+str(unique_bioshaker[shaker]))
 		plt.tight_layout()
-
+	plt.legend(bbox_to_anchor=(1.2, 1.1))
 
 	#Use the linear models to correct the volume loss by bioshaker
 	df_gr_comp = pd.DataFrame()
@@ -303,6 +403,11 @@ def compensation_lm(cor_df, df_gr) : #done
 		df_gr_comp["Correlation"] = lm_eq[pos][0]*df_gr_comp["time_hours"]+lm_eq[pos][1]
 		df_gr_comp["Measurement"] = df_gr_comp["Measurement"]*df_gr_comp["Correlation"]
 		df_gr_comp_out = df_gr_comp_out.append(df_gr_comp)
+
+	plt.savefig("lm_volume_loss.png", dpi=250)
+	plt.close()
+	print("Volume loss correction : DONE")
+
 	return fig, df_gr_comp_out
 
 
@@ -354,10 +459,50 @@ def reshape_dataframe(df_gr, flag_species = False, flag_bioshaker = False) :
 		df_gr_final = pd.concat([df_gr_final,column2], ignore_index=False, axis=1)
 
 
-	if flag_species == False :
-
+	if flag_species == False and flag_bioshaker == False:
 		return df_gr_final
 
+	elif flag_species == False and flag_bioshaker == True:
+
+		df_gr_temp["Sample_ID"] = df_gr_temp["Sample_ID"]+"_"+df_gr_temp["Species"]
+		unique_species = df_gr_temp["Species"].unique()
+		unique_bioshaker = (df_gr_temp["Sample_ID"].str[0:3]).unique()
+		cols = ["Sample_ID", "Measurement", "time_hours", "Species"]	#relevant variables
+		df_gr = df_gr_temp[cols]
+
+		list_df_species = []
+		df_gr_final_list = []
+		df_temp = pd.DataFrame()
+
+		for pos in range(len(unique_bioshaker)) :
+
+			df_temp = df_gr[df_gr_temp.Sample_ID.str.contains(unique_bioshaker[pos])]
+			list_df_species.append(df_temp)
+
+			for df in list_df_species :
+
+				unique_IDs = df["Sample_ID"].unique()
+				unique_times = df["time_hours"].unique()
+				df_fin = pd.DataFrame()
+				#Initialize new dataframe
+				#An ID column is created for the measurement of every sample and another column timeID is created to relate it to the times
+				
+				for i in range(len(unique_IDs)) :
+
+					m_list = df.loc[df["Sample_ID"] == unique_IDs[i], "Measurement"].tolist()
+					column1 = pd.DataFrame({unique_IDs[i] : m_list})
+					t_list = df.loc[df["Sample_ID"] == unique_IDs[i], "time_hours"].tolist()
+					column2 = pd.DataFrame({"time_"+unique_IDs[i] : t_list})
+					df_fin = pd.concat([df_fin,column1], ignore_index=False, axis=1)
+					df_fin = pd.concat([df_fin,column2], ignore_index=False, axis=1)
+					column1 = pd.DataFrame()
+					column2 = pd.DataFrame()
+
+				df_gr_final_list.append(df_fin)
+
+		return df_gr_final, df_gr_final_list
+
+
 	else :
 	
 		df_gr_temp["Sample_ID"] = df_gr_temp["Sample_ID"]+"_"+df_gr_temp["Species"]
@@ -380,7 +525,6 @@ def reshape_dataframe(df_gr, flag_species = False, flag_bioshaker = False) :
 					df_temp = df_gr[df_gr.Species.str.contains(unique_species[pos1])]
 					df_temp = df_temp[df_temp.Sample_ID.str.contains(unique_bioshaker[pos2])]
 					df_temp = df_temp.drop(columns=["Species"])
-
 					list_df_species.append(df_temp)
 
 		elif flag_bioshaker == False :
@@ -447,7 +591,6 @@ def gr_estimation(df_gr_final) :
 
 	df_annotations["Parameter"] = ["Start", "End", "Slope", "Intercept", "n0", "SNR", "rank"]
 
-
 	for col in range(len(colnames)):
 
 		#Series format for process_curve input
@@ -458,7 +601,6 @@ def gr_estimation(df_gr_final) :
 
 			#Some samples are too noise to handle by the croissance library and raise an error
 			gr_estimation = process_curve(my_series)
-
 		except :
 
 			#For those samples that raise errors, the outliers are removed and a series is returned
@@ -470,7 +612,7 @@ def gr_estimation(df_gr_final) :
 		#Dataframe generation with outlier free data
 		df_temp = pd.DataFrame({colnames[col]+'_data':est_series.index, colnames[col]+"_time":est_series.values})
 		df_data_series = pd.concat([df_data_series,df_temp], ignore_index=False, axis=1)
-
+		
 		#--Annotated growth curve--
 
 		if colnames[col] in errors or len(gr_estimation[2]) == 0 :
@@ -482,8 +624,6 @@ def gr_estimation(df_gr_final) :
 
 				pass
 
-
-
 		else :
 			
 			#Initialize objects
@@ -511,7 +651,15 @@ def gr_estimation(df_gr_final) :
 			#Append Annotations to returned dataframe
 			df_temp_annotations[colnames[col]] = list_annotations
 			df_annotations = pd.concat([df_annotations , df_temp_annotations], ignore_index=False, axis=1)
-			
+
+	if len(errors) > 0 and len(list_annotations) != 0:
+
+		print("\nSome samples linear phase could not be estimated, the list of non estimated samples can be seen in the estimation_logfile.txt\n")
+
+	elif len(errors) > 0 and len(list_annotations) == 0:
+
+		print("\nNone of the samples linear phase could be estimated due to noisy data\n")
+
 	return df_data_series, df_annotations, errors
 
 
@@ -535,11 +683,10 @@ def estimation_writter(df_data_series, df_annotations, error_list) :
 	df_data_series.to_excel(r'Data_series.xlsx', header = True,index = False)
 	df_annotations.to_excel(r'Annotations.xlsx', header = True, index = False)
 
-	outfile = open('errors.txt', 'w')
-	
-
+	outfile = open('estimations_logfile.txt', 'w')
 
 	outfile.write("List of non estimated samples :\n")
+	
 	for error in error_list :
 
 		outfile.write(error+"\n")
@@ -553,7 +700,7 @@ def estimation_writter(df_data_series, df_annotations, error_list) :
 # ----- PLOTTING GROWTH RATE CURVE -----
 
 
-def gr_plots(df, sample, color_ = None, ind = False, legend_ = "bioshaker", title_ = "species" ) :
+def gr_plots(df, sample,interpolationplot, color_ = None, ind = False, legend_ = "bioshaker", title_ = "species") :
 	'''Generates a growth curve plot for a given series for common species, returns the plot.
 	
 	Args:
@@ -571,18 +718,26 @@ def gr_plots(df, sample, color_ = None, ind = False, legend_ = "bioshaker", titl
 	
 	#Individual plots
 	if ind == True :
-		x_new = np.linspace(df["time"].min(),df["time"].max(),500)
-		a_BSpline = interpolate.make_interp_spline(df["time"], df[sample])
-		y_new = a_BSpline(x_new)
-		plt.figure()
-		plt.plot(x_new,y_new)
-		fig = plt.scatter(df["time"],df[sample],5, facecolor=(.18, .31, .31))
+
+		if interpolationplot == True :
+			try :
+				x_new = np.linspace(df["time"].min(),df["time"].max(),500)
+				a_BSpline = interpolate.make_interp_spline(df["time"], df[sample])
+				y_new = a_BSpline(x_new)
+				plt.figure()
+				plt.plot(x_new,y_new)
+
+			except ValueError:
+				pass
+
+		fig = plt.scatter(df["time"],df[sample],5, facecolor=(.18, .31, .31), label=legend_label)
 		plt.ylabel('Absorbance (OD)', fontname="Arial", fontsize=12)
 		plt.xlabel('Time (h)', fontname="Arial", fontsize=12)
 		plt.title("Growth rate curve of "+str(sample), fontname="Arial", fontsize=12)
-		#plt.legend(bbox_to_anchor=(1.05, 1.0), loc='upper left')
+		plt.legend(bbox_to_anchor=(1.05, 1.0), loc='upper left')
 		plt.tight_layout()
-		return fig, plt.savefig(sample+"_GR_curve.png")
+
+		return fig, plt.savefig(str(sample)+"_GR_curve.png")
 
 	#Create plots by combined by species
 	elif ind == False :
@@ -594,7 +749,7 @@ def gr_plots(df, sample, color_ = None, ind = False, legend_ = "bioshaker", titl
 
 		elif legend_ == "species" :
 
-			legend_label = sample[-6:]
+			legend_label = re.search(r"^\S{3}.\d?_?\S{2,3}_(\S+)", sample).group(1)
 
 		else :
 
@@ -607,22 +762,26 @@ def gr_plots(df, sample, color_ = None, ind = False, legend_ = "bioshaker", titl
 
 		elif title_ == "species" :
 
-			title_label = sample[-6:]
+			title_label = re.search(r"^\S{3}.\d?_?\S{2,3}_(\S+)", sample).group(1)
 
 		elif title_ == "species_bioshaker" :
 
-			title_label = sample[:3]+"_"+sample[-6:]
+			title_label = sample[:3]+"_"+re.search(r"^\S{3}.\d?_?\S{2,3}_(\S+)", sample).group(1)
 		
 		else :
-			
 			pass
 
+		if interpolationplot == True :
+			try:
+				x_new = np.linspace(df["time"].min(),df["time"].max(),500)
+				a_BSpline = interpolate.make_interp_spline(df["time"], df[sample])
+				y_new = a_BSpline(x_new)
+				plt.plot(x_new,y_new , color = color_, label=legend_label)
 
-		x_new = np.linspace(df["time"].min(),df["time"].max(),500)
-		a_BSpline = interpolate.make_interp_spline(df["time"], df[sample])
-		y_new = a_BSpline(x_new)
-		plt.plot(x_new,y_new , color = color_, label=legend_label)
-		fig = plt.scatter(df["time"],df[sample],5, facecolor=(.18, .31, .31))
+			except ValueError:
+				pass
+		
+		fig = plt.scatter(df["time"],df[sample],5, facecolor=(.18, .31, .31), color = color_, label=legend_label)
 		plt.ylabel('Absorbance (OD)', fontname="Arial", fontsize=12)
 		plt.xlabel('Time (h)', fontname="Arial", fontsize=12)
 		plt.title("Growth rate curve of "+str(title_label), fontname="Arial", fontsize=12)
@@ -650,7 +809,7 @@ def stats_summary(df_annotations) :
 	summary_df =  pd.DataFrame()
 
 	#Species and bioshaker labels
-	species_list= [re.sub(r'\S+[.]\S+[_](\S+)', r'\1', i) for i in (df_annotations.columns.values[1:])]
+	species_list= [re.sub(r'\S+[.]?\S+[_](\S+)', r'\1', i) for i in (df_annotations.columns.values[1:])]
 	x = slice(0, 3) 
 	bioshaker_list = [i[x] for i in (df_annotations.columns.values[1:])]
 	summary_df["species"] =  species_list
@@ -688,6 +847,48 @@ def stats_summary(df_annotations) :
 
 	return summary_df, mean_df_species, mean_df_bs
 
+def stats_plot(summary_df):
+	'''Box plots of annotation growth rate parameters by species and bioshaker
+	Args: 
+	summary_df : dataframe containing the annotation parameters
+	Return:
+	call: string with status of plots creation
+	'''
+
+	if len(summary_df.index) == 0:
+		call = "Summary statistics plots not computed: No parameters were estimated and thus no plots can be shown"
+		return call
+
+	else :
+		plt.close()
+		sns.boxplot(x="species", y="start", hue="bioshaker", data=summary_df, palette="Pastel1")
+		plt.savefig("start_boxplot",  dpi=250)
+		plt.close()
+
+		plot_end = sns.boxplot(x="species", y="end", hue="bioshaker", data=summary_df, palette="Pastel1")
+		plt.savefig("end_boxplot",  dpi=250)
+		plt.close()
+
+		plot_slope = sns.boxplot(x="species", y="slope", hue="bioshaker", data=summary_df, palette="Pastel1")
+		plt.savefig("slope_boxplot",  dpi=250)
+		plt.close()
+
+		plot_intercep = sns.boxplot(x="species", y="intercep", hue="bioshaker", data=summary_df, palette="Pastel1")
+		plt.savefig("intercept_boxplot",  dpi=250)
+		plt.close()
+
+		plot_n0 = sns.boxplot(x="species", y="n0", hue="bioshaker", data=summary_df, palette="Pastel1")
+		plt.savefig("n0_boxplot",  dpi=250)
+		plt.close()
+
+		plot_SNR = sns.boxplot(x="species", y="SNR", hue="bioshaker", data=summary_df, palette="Pastel1")
+		plt.savefig("SNR_boxplot",  dpi=250)
+		plt.close()
+
+		call = "Summary statistics : DONE"
+
+		return call
+
 
 
 def exponential(x, intercep, slope, n0):
@@ -696,8 +897,6 @@ def exponential(x, intercep, slope, n0):
 	return estimation
 
 
-
-
 def interpolation(od_measurements, df_annotations, mean_df_bs):
 	'''Interpolates the values of given od readings and returns growth rate measurements.
 	
@@ -782,25 +981,3 @@ def interpolation(od_measurements, df_annotations, mean_df_bs):
 	od_measurements.to_excel(r'interpolation_results.xlsx', header = True,index = False)
 
 	return od_measurements
-		
-
-	
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
